import unittest

from is_allowed import is_allowed
from utils import logger

logger.info(">>> START UNITTEST <<<")


class IsAllowedTest(unittest.TestCase):
    def test_not_allowed_inputs(self):
        self.assertEqual(is_allowed('123456'), 'NOT_ALLOWED')
        self.assertEqual(is_allowed('12345G'), 'NOT_ALLOWED')
        self.assertEqual(is_allowed('12345'), 'NOT_ALLOWED')
        self.assertEqual(is_allowed('12L3456'), 'NOT_ALLOWED')
        self.assertEqual(is_allowed('1234M56'), 'NOT_ALLOWED')
        self.assertEqual(is_allowed('100000000000000000'), 'NOT_ALLOWED')
        self.assertEqual(is_allowed('123MM456'), 'NOT_ALLOWED')
        self.assertEqual(is_allowed('1234LL56'), 'NOT_ALLOWED')

    def test_not_valid_inputs(self):
        # All ascii Non-Alphanumeric Printable Characters
        for i in '!\"#$%&\'()*+-./:;<=>?@[\\]^_{|}~':
            self.assertEqual(is_allowed(f'{i}'), 'NOT_VALID')
        self.assertEqual(is_allowed('asf78*'), 'NOT_VALID')
        self.assertEqual(is_allowed('asf%78'), 'NOT_VALID')
        self.assertEqual(is_allowed(''), 'NOT_VALID')

    def test_allowed_inputs(self):
        self.assertEqual(is_allowed('123HJK'), 'ALLOWED')
        self.assertEqual(is_allowed('123GHJK'), 'ALLOWED')
        self.assertEqual(is_allowed('123567O'), 'ALLOWED')


if __name__ == '__main__':
    unittest.main()



import ocrspace
from ocrspace.main import Language

from config import api_key
from utils import logger


def ocr(file: str) -> str:
    """
    :params: file as a path string
    :return: str (License plate number)
    """
    api = ocrspace.API(
        endpoint='https://api.ocr.space/parse/image', api_key=api_key)
    try:
        filename = file

        result = api.ocr_file(open(filename, 'rb')).splitlines()

        return result[0] if result != [] else ''

    except Exception as e:
        logger.warning(e, exc_info=True)
        raise Exception(
            "There is something wrong with the api please check log for farther understanding")



import db
from is_allowed import is_allowed
from ocr_api import ocr


class ParkingLot():
    def __init__(self, user: str = 'postgres', password: str = 'password') -> None:
        cur = db.db_establish_connection(user, password)
        db.db_create_database(cur)
        db.db_create_table(database='parkinglot')

    @staticmethod
    def check(img: str) -> str:
        license = ocr(img)
        return is_allowed(license)

    @staticmethod
    def db_insert(license: str, status: str):
        db.db_insert(license, status)


def main():
    pass


if __name__ == '__main__':
    main()



import logging
import os
from datetime import datetime

# AppData path for log saving
env = os.getenv('APPDATA') + '\\parkinglot_log.txt'

# Logger instance
LOG_FORMAT = "%(levelname)s %(asctime)s - %(message)s"
logging.basicConfig(
    handlers=[logging.FileHandler(env, 'w', 'utf-8')],
    format=LOG_FORMAT,
    level=logging.DEBUG)
logger = logging.getLogger()

# Datetime instance of now
now = datetime.now().strftime("%d-%m-%Y %H:%M:%S")



import regex as re

from db import db_insert
from utils import logger, now


def is_allowed(license: str) -> str:
    """:return: ALLOWED/NOT_ALLOWED/NOT_VALID to enter the parking lot"""

    logger.info(f'Function is_allowed is running with {license = }')

    enum = ['ALLOWED', 'NOT_ALLOWED', 'NOT_VALID']
    state = enum[0]

    try:
        # Check if there is any (non-digit || non-letter) in license charecters
        if re.sub(r'[a-zA-Z0-9]', '', license):
            state = enum[2]
            return state

        # Check if last charecter in license is 6 or G
        if license[-1] in ('6', 'G'):
            state = enum[1]
            db_insert(license, state)
            return state

        # Check rather there is ('L' || 'M') in license charecters
        if 'L' in license or 'M' in license:
            state = enum[1]
            db_insert(license, state)
            return state

        # Check if license charecters are all digits
        if False not in [*map(str.isdigit, license)]:
            state = enum[1]
            db_insert(license, state)
            return state

    except Exception as err:
        logger.warning(
            f'License is an empty string! \n{str(err)}', exc_info=True)
        state = enum[2]
        return state

    finally:
        logger.info(
            f'Function is_allowed done checking {license = } with status: {state}')

    db_insert(license, state)
    return state


def main():
    pass


if __name__ == '__main__':
    main()



from setuptools import setup, find_packages


VERSION = '0.0.3'
DESCRIPTION = 'ParkingLot is a Python service imitating a parking lot like system.'
LONG_DESCRIPTION = 'The service indicates rather a vehicle allowed or not allowed to enter the parking lot.'

# Setting up
setup(
    name="parkinglot",
    version=VERSION,
    author="PsychoRover (Gil Chen)",
    author_email="<Gtorrent12@gmail.com>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=LONG_DESCRIPTION,
    packages=find_packages(),
    install_requires=['psycopg2', 'ocrspace'],
    keywords=['python', 'parking', 'lot', 'moon', 'parkinglot'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)



import psycopg2 as pg2

from utils import logger, now


def db_establish_connection(user: str = 'postgres', password: str = 'password', database=None) -> 'pg2.cursor':
    """:description: Establishing connection to PostgresDB"""

    try:
        conn = pg2.connect(user=user, password=password, database=database)
        conn.autocommit = True
        cur = conn.cursor()
    except Exception as e:
        logger.fatal(e, exc_info=True)
        raise Exception(
            "It seems like you have some problem with your database, please proceed reading the log for farther understading")
    finally:
        logger.info("db_establish_connection status: Done!")

    return cur


def db_create_database(cur: 'pg2.cursor') -> None:
    """:description: Creating 'parkinglot' database if not exist"""

    try:
        cur.execute("SELECT datname FROM pg_database;")
        if 'parkinglot' not in sum(cur.fetchall(), ()):
            cur.execute(
                """
                CREATE DATABASE parkinglot
                WITH
                    ENCODING = 'UTF8'
                    CONNECTION LIMIT = 100
                    ALLOW_CONNECTIONS = true;
                """)
            logger.info("Database 'parkinglot' was created successfully!")
        else:
            logger.info("Database 'parkinglot' already exist")

    except Exception as e:
        logger.fatal(e, exc_info=True)
        raise Exception(
            "It seems like you have some problem with your database, please proceed reading the log for farther understading")

    finally:
        logger.info('db_create_database status: Done!')
        cur.close()


def db_create_table(user: str = 'postgres', password: str = 'password', database=None) -> None:
    """:description: Creating 'entrances' table if not exist"""

    # Establish connection with relevant database
    cur = db_establish_connection(user, password, database)

    try:
        cur.execute(
            """
            SELECT tablename FROM pg_catalog.pg_tables
            WHERE schemaname = 'public';
            """)

        if 'entrances' not in sum(cur.fetchall(), ()):
            cur.execute(
                """
                CREATE TABLE entrances(
                    id SERIAL PRIMARY KEY,
                    license_number VARCHAR(255) NOT NULL,
                    is_allowed VARCHAR(255) NOT NULL,
                    time TIMESTAMP NOT NULL
                );
                """
            )
            logger.info("Table 'entrances' was created successfully!")
        else:
            logger.info("Table 'entrances' already exist")

    except Exception as e:
        logger.fatal(e, exc_info=True)
        raise Exception(
            "It seems like you have some problem with your database, please proceed reading the log for farther understading")

    finally:
        logger.info('db_create_table status: Done!')
        cur.close()


def db_insert(liscense: str, status: str) -> None:
    """:description: Inserting values to DB"""

    # Establish connection with relevant database
    cur = db_establish_connection(database='parkinglot')

    try:
        cur.execute(
            f"""
            INSERT INTO entrances(license_number, is_allowed, time)
            VALUES
            ('{liscense}', '{status}', to_timestamp('{now}', 'dd-mm-yyyy hh24:mi:ss'))
            """
        )

    except Exception as e:
        logger.fatal(e, exc_info=True)
        raise Exception(
            "It seems like you have some problem with your database, please proceed reading the log for farther understading")

    finally:
        logger.info("db_insert status: Done!")
        cur.close()


def main():
    pass


if __name__ == '__main__':
    main()
